import os
from pathlib import Path
from typing import Union, List

from pydantic import BaseModel

# HERE = os.path.abspath(os.path.dirname(__file__))
HERE = os.getcwd()


class Provider(BaseModel):
    """
    This holds the information about the provider
    """

    name: Union[str, None] = None
    project: Union[str, None] = None
    file_name: Union[str, None] = None
    api_key: Union[str, None] = None
    account_id: Union[str, None] = None
    email_address: Union[str, None] = None
    path: Union[str, None] = None


def check_hugo_file() -> Provider:
    """
    Checks for ``config.yaml`` or ``config.toml``, if exists then it checks for ``uhugo`` key

    :return Provider: An object with ``name`` and ``file_name``
    """

    path = Path(HERE, "config.toml")
    if not path.exists():
        path = Path(HERE, "config.yaml")
        if not path.exists():
            return Provider()
        else:
            import yaml

            try:
                from yaml import CLoader as Loader, CDumper as Dumper
            except ImportError:
                from yaml import Loader, Dumper
            with open(path) as f:
                data = yaml.load(f, Loader=Loader)
    else:
        import toml

        with open(path) as f:
            data = toml.load(f)

    return Provider(**data["uhugo"])


def check_providers_fs() -> List[Provider]:
    """
    Checks file system for any providers that matches the list

    :return: A Provider
    """

    files = ["netlify.yaml", "vercel.json", "netlify.toml"]
    providers: List[Provider] = []

    for file in files:
        path = Path(HERE, file)
        if path.exists():
            providers.append(Provider(**{"name": path.name.split(".")[0], "path": path.__str__()}))

    return providers



import subprocess
from dataclasses import dataclass

from packaging import version


@dataclass
class Hugo:
    """Dataclass for Hugo"""

    path: str
    exists: bool
    version: version.Version


def hugo_version_cmd() -> bytes:
    """
    Returns Hugo version

    :return: Hugo version
    """

    return subprocess.check_output(["hugo", "version"]).strip()



import os.path
import platform
import tempfile

import requests
from rich.progress import Progress, TextColumn, BarColumn, DownloadColumn, TransferSpeedColumn, TimeRemainingColumn

progress = Progress(
    TextColumn("[bold blue]{task.fields[filename]}", justify="right"),
    BarColumn(),
    "[progress.percentage]{task.percentage:>3.1f}%",
    "•",
    DownloadColumn(),
    "•",
    TransferSpeedColumn(),
    "•",
    TimeRemainingColumn(),
    transient=True,
)


def download_hugo_zip(version: str, os_type: str = None, download_to: str = None) -> str:
    """
    Download the Hugo file to temp folder.

    :param os_type: OS type
    :param version: Version number to download
    :param download_to: Path to download to
    """

    if not os_type:
        os_type = platform.system()

    if not download_to:
        download_to = tempfile.gettempdir()

    download_to = os.path.join(download_to, f"hugo_{version}")
    with progress:
        task_id = progress.add_task("download", filename=f"hugo_{version}", start=False)

        with open(download_to, "wb") as file:
            if os_type == "Darwin":
                response = requests.get(
                    f"https://github.com/gohugoio/hugo/releases/download/v{version}/"
                    f"hugo_extended_{version}_darwin-universal.tar.gz",
                    stream=True,
                )
            elif os_type == "Windows" or os_type == "nt":
                response = requests.get(
                    f"https://github.com/gohugoio/hugo/releases/download/v{version}/"
                    f"hugo_extended_{version}_windows-amd64.zip",
                    stream=True,
                )
            elif os_type == "posix" or os_type == "Linux":
                response = requests.get(
                    f"https://github.com/gohugoio/hugo/releases/download/v{version}/"
                    f"hugo_extended_{version}_linux-amd64.tar.gz",
                    stream=True,
                )
            else:
                raise OSError(f"{os_type} not supported.")

            if response.headers.get("Status") == "404 Not Found":
                raise requests.exceptions.HTTPError("File not found")

            total_length = int(response.headers.get("content-length"))

            if total_length is None:
                progress.console.print(f"- Downloading Hugo v{version}")
                file.write(response.content)
                progress.update(task_id, completed=True)
                progress.console.print("- Hugo downloaded")
            else:
                progress.console.print(f"- Downloading Hugo v{version}")
                progress.update(task_id, total=total_length)
                progress.start_task(task_id)
                for data in response.iter_content(chunk_size=4096):
                    file.write(data)
                    progress.update(task_id, advance=len(data))
                progress.console.print("- Hugo downloaded")

    return download_to



from typing import Text

from uhugo.post_install.providers import ProviderBase


class Vercel(ProviderBase):
    """
    Vercel provider
    """

    def update_config_file(self, file_name: Text):
        """
        Updates ``vercel.json`` file with Hugo's version

        :param file_name: Path of ``vercel.json``
        :return:
        """
        pass

    def update_api(self, key: Text):
        """
        Updates Cloudflare Pages environment variable of ``HUGO_VERSION``.

        :param key: API key
        :return:
        """
        pass
